package de.ozgcloud.xta.client;

import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import de.ozgcloud.xta.client.model.XtaIdentifier;
import de.ozgcloud.xta.client.model.XtaMessage;
import de.ozgcloud.xta.client.model.XtaMessageMetaData;

record FetchMessageParameter(
		XtaIdentifier clientIdentifier,
		Consumer<XtaMessage> processMessage,
		Set<String> viewedMessageIds
) {

	public FetchMessageParameter withViewedMessageIdsFrom(List<XtaMessageMetaData> messageMetaData) {
		return new FetchMessageParameter(this.clientIdentifier, this.processMessage,
				collectSetOfViewedMessageIds(messageMetaData, this.viewedMessageIds));
	}

	private Set<String> collectSetOfViewedMessageIds(List<XtaMessageMetaData> messageMetaData, Set<String> processedMessageIds) {
		return Stream.concat(
				processedMessageIds.stream(),
				messageMetaData.stream()
						.map(XtaMessageMetaData::messageId)
		).collect(Collectors.toSet());
	}

	public boolean hasMessageAlreadyBeenViewed(XtaMessageMetaData messageMetaData) {
		return viewedMessageIds.contains(messageMetaData.messageId());
	}

}
