package de.ozgcloud.xta.client.mapper;

import org.apache.cxf.ws.addressing.AttributedURIType;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.xta.client.model.XtaFile;
import de.ozgcloud.xta.client.model.XtaIdentifier;
import de.ozgcloud.xta.client.model.XtaMessage;
import de.ozgcloud.xta.client.model.XtaMessageMetaData;
import genv3.de.xoev.transport.xta.x211.ContentType;
import genv3.de.xoev.transport.xta.x211.GenericContentContainer;
import genv3.de.xoev.transport.xta.x211.LookupServiceRequest;
import genv3.de.xoev.transport.xta.x211.LookupServiceType;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxCloseRequestType;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxFetchRequest;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxStatusListRequestType;
import genv3.eu.osci.ws.x2014.x10.transport.KeyCodeType;
import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import genv3.eu.osci.ws.x2014.x10.transport.PartyIdentifierType;
import genv3.eu.osci.ws.x2014.x10.transport.PartyType;
import genv3.eu.osci.ws.x2014.x10.transport.QualifierType;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR
)
public interface RequestMapper {
	String MESSAGE_TYPE_LIST_URI = "urn:de:payloadSchema:elementName";
	String MESSAGE_TYPE_LIST_VERSION_ID = "1.0";
	String IDENTIFIER_TYPE = "xoev";
	String CLOSE_REQUEST_ID = "1";

	@Mapping(target = "securityToken", ignore = true)
	PartyType mapPartyTypeFromIdentifier(XtaIdentifier identifier);

	@Mapping(target = "type", constant = IDENTIFIER_TYPE)
	PartyIdentifierType mapPartyIdentifierTypeFromIdentifier(XtaIdentifier value);

	@Mapping(target = "maxListItems", source = "maxListItems")
	@Mapping(target = "listForm", constant = "MessageMetaData")
	@Mapping(target = "msgSelector", ignore = true)
	MsgBoxStatusListRequestType mapMsgBoxStatusListRequestTypeFromMaxListItems(Integer maxListItems);

	@Mapping(target = "qualifier", source = ".")
	@Mapping(target = "msgIdentification.messageID.value", source = "messageId")
	@Mapping(target = "originators.author.identifier", source = "authorIdentifier")
	@Mapping(target = "destinations.reader.identifier", source = "readerIdentifier")
	@Mapping(target = "msgSize", source = "messageSize")
	@Mapping(target = "testMsg", ignore = true)
	@Mapping(target = "deliveryAttributes", ignore = true)
	@Mapping(target = "messageProperties", ignore = true)
	MessageMetaData mapMessageMetaDataFromXtaMessageMetaData(XtaMessageMetaData xtaMessageMetaData);

	@Mapping(target = "encryptedData", expression = "java(null)")
	@Mapping(target = "contentContainer.message", source = "messageFile")
	@Mapping(target = "contentContainer.attachment", source = "attachmentFiles")
	GenericContentContainer mapGenericContentContainer(XtaMessage xtaMessage);

	@Mapping(target = "value", source = "content")
	@Mapping(target = "filename", source = "name")
	@Mapping(target = "lang", source = "language")
	ContentType mapContentType(XtaFile xtaFile);

	@Mapping(target = "lastMsgReceived", expression = "java( List.of(mapAttributedURIType(messageId)) )")
	@Mapping(target = "msgBoxRequestID", constant = CLOSE_REQUEST_ID)
	MsgBoxCloseRequestType mapMsgBoxCloseRequestType(String messageId);

	@Mapping(target = "otherAttributes", ignore = true)
	AttributedURIType mapAttributedURIType(String value);

	@Mapping(target = "lookupServiceRequestList", expression = "java( List.of( mapLookupServiceRequestList(service, reader )) )")
	LookupServiceRequest mapLookupServiceRequest(String service, XtaIdentifier reader);

	@Mapping(target = "lookupService", expression = "java( mapLookupServiceType(service, reader) )")
	LookupServiceRequest.LookupServiceRequestList mapLookupServiceRequestList(String service, XtaIdentifier reader);

	LookupServiceType mapLookupServiceType(String serviceType, XtaIdentifier reader);

	@Mapping(target = "msgSelector.messageID", expression = "java( List.of( mapAttributedURIType(string) ) )")
	@Mapping(target = "msgPart", ignore = true)
	MsgBoxFetchRequest mapMsgBoxFetchRequest(String messageId);

	@Mapping(target = "subject", ignore = true)
	@Mapping(target = "messageType", source = ".")
	@Mapping(target = "businessScenario", source = ".", qualifiedByName = "mapBusinessScenario")
	QualifierType mapQualifierType(XtaMessageMetaData messageMetaData);

	@Mapping(target = "code", source = "messageTypeCode")
	@Mapping(target = "name", ignore = true)
	@Mapping(target = "payloadSchema", source = "messageTypePayloadSchema")
	@Mapping(target = "listURI", constant = MESSAGE_TYPE_LIST_URI)
	@Mapping(target = "listVersionID", constant = MESSAGE_TYPE_LIST_VERSION_ID)
	QualifierType.MessageType mapQualifierTypeMessageType(XtaMessageMetaData messageMetaData);

	@Named("mapBusinessScenario")
	default QualifierType.BusinessScenario mapBusinessScenario(XtaMessageMetaData messageMetaData) {
		return messageMetaData.businessScenarioListUri() != null
				? mapDefinedBusinessScenario(messageMetaData)
				: mapUndefinedBusinessScenario(messageMetaData);
	}

	@Mapping(target = "defined", expression = "java(null)")
	@Mapping(target = "undefined", source = "businessScenarioCode")
	QualifierType.BusinessScenario mapUndefinedBusinessScenario(XtaMessageMetaData messageMetaData);

	@Mapping(target = "defined", source = ".", qualifiedByName = "mapDefinedBusinessScenarioCode")
	@Mapping(target = "undefined", expression = "java(null)")
	QualifierType.BusinessScenario mapDefinedBusinessScenario(XtaMessageMetaData messageMetaData);

	@Named("mapDefinedBusinessScenarioCode")
	@Mapping(target = "code", source = "businessScenarioCode")
	@Mapping(target = "name", ignore = true)
	@Mapping(target = "listURI", source = "businessScenarioListUri")
	@Mapping(target = "listVersionID", source = "businessScenarioListVersionId")
	KeyCodeType mapDefinedBusinessScenarioCode(XtaMessageMetaData messageMetaData);
}
