package de.ozgcloud.xta.client.model;

import java.math.BigInteger;

import jakarta.activation.DataHandler;
import jakarta.annotation.Nullable;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.PositiveOrZero;

import lombok.Builder;

/**
 * Represents a file in the XTA context.
 *
 * <p>
 * <small>Parameter documentation strings translated from <i>XTA-Webservice-Datentypen.xsd</i>.</small>
 * </p>
 *
 * @param content            A data handler of the file content. <p>This field is required.</p>
 * @param contentType        This attribute specifies the MIME type of the contained content, so it includes entries like {@code text/xml},
 *                           {@code text/plain}, {@code application/gzip}, or {@code application/pdf}. <p>This field is required as it is particularly
 *                           important information (handled analogously in email).</p>
 * @param contentDescription Description of the content, e.g., 'Offer' or 'Invoice'. <p>This field is optional.</p>
 * @param encoding           Character set that was used for encoding the content. <p>This field is optional.</p>
 * @param name               Filename of the data source, if the content was taken from a file. E.g.: For the transmission of xdomea messages,
 *                           this attribute is mandatory. <p>This field is recommended as it is required for xdomea.</p>
 * @param id                 Provides the ability to reference the content via, for example, a running number. <p>This field is optional and will be
 *                           automatically set by the XTA server.</p>
 * @param language           Language of the file content, for instance, {@code en-US} or {@code de-DE}, see <a
 *                           href="http://www.ietf.org/rfc/rfc3066.txt">RFC 3066</a>. <p>This field is optional.</p>
 * @param size               Size of the file content in bytes.
 *
 *                           <p>
 *                           This field is optional and will be automatically set by the XTA server.
 *                           </p>
 */
@Builder(toBuilder = true)
public record XtaFile(
		@NotNull DataHandler content,
		@NotBlank String contentType,
		@Nullable String contentDescription,
		@Nullable String encoding,
		@NotBlank String name,
		@Nullable String id,
		@Nullable String language,
		@Nullable @PositiveOrZero BigInteger size
) {
}
