package de.ozgcloud.xta.client.xdomea.reader;

import java.util.stream.IntStream;
import java.util.stream.Stream;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.ozgcloud.xta.client.exception.XtaClientRuntimeException;
import lombok.Builder;
import lombok.RequiredArgsConstructor;

@Builder
@RequiredArgsConstructor
public class XmlValueReader {

	private final XPathExpression xPathExpression;

	public Stream<String> readNonEmptyTexts(Document xmlDocument) {
		return getTextsFromNodes(queryNodeList(xmlDocument, xPathExpression))
				.filter(s -> !s.isEmpty());
	}

	private NodeList queryNodeList(Document xmlDocument, XPathExpression xPathExpression) {
		try {
			return (NodeList) xPathExpression
					.evaluate(
							xmlDocument.getDocumentElement(),
							XPathConstants.NODESET
					);
		} catch (XPathExpressionException e) {
			throw new XtaClientRuntimeException("Failed to execute xpath search!", e);
		}
	}

	private Stream<String> getTextsFromNodes(NodeList nodeList) {
		return streamNodeList(nodeList)
				.map(Node::getTextContent)
				.map(String::trim);
	}

	private Stream<Node> streamNodeList(NodeList nodeList) {
		return IntStream.range(0, nodeList.getLength())
				.mapToObj(nodeList::item);
	}

}
