package de.ozgcloud.xta.client.xdomea.reader;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import jakarta.activation.DataHandler;

import de.ozgcloud.xta.client.exception.XtaClientRuntimeException;
import lombok.Builder;
import lombok.RequiredArgsConstructor;

@Builder
@RequiredArgsConstructor
public class ZipFileEntryReader {

	public <T> T mapEntryData(DataHandler zipFileContent, String entryName, Function<InputStream, T> inputStreamConsumer) {
		try (var zipInputStream = new ZipInputStream(zipFileContent.getInputStream())) {
			ZipEntry entry;
			while ((entry = zipInputStream.getNextEntry()) != null) {
				if (entryName.equals(entry.getName())) {
					return inputStreamConsumer.apply(zipInputStream);
				}
				zipInputStream.closeEntry();
			}
			throw new XtaClientRuntimeException("Failed to find zip entry '%s'!".formatted(entryName));
		} catch (IOException e) {
			throw new XtaClientRuntimeException("Failed reading zip file!", e);
		}
	}

	public List<String> getEntryNames(DataHandler zipFileContent) {
		var entryNames = new ArrayList<String>();
		try (var zipInputStream = new ZipInputStream(zipFileContent.getInputStream())) {
			ZipEntry entry;
			while ((entry = zipInputStream.getNextEntry()) != null) {
				entryNames.add(entry.getName());
				zipInputStream.closeEntry();
			}
			return entryNames;
		} catch (IOException e) {
			throw new XtaClientRuntimeException("Failed reading zip file!", e);
		}
	}
}
