package de.ozgcloud.xta.client;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.xta.client.config.XtaClientConfig;
import de.ozgcloud.xta.client.config.XtaConfigValidator;
import de.ozgcloud.xta.client.core.XtaClientService;
import de.ozgcloud.xta.client.core.XtaClientServiceFactory;
import de.ozgcloud.xta.client.core.XtaExceptionHandler;
import de.ozgcloud.xta.client.core.XtaExceptionHandlerFactory;
import lombok.SneakyThrows;

class XtaClientFactoryTest {

	@Mock
	private XtaConfigValidator configValidator;
	@Mock
	private XtaClientServiceFactory xtaClientServiceFactory;
	@Mock
	private XtaClientConfig config;
	@Mock
	private XtaExceptionHandlerFactory xtaExceptionHandlerFactory;

	@InjectMocks
	private XtaClientFactory factory;

	@DisplayName("create")
	@Nested
	class TestCreate {
		@Mock
		private XtaClientService service;
		@Mock
		private XtaExceptionHandler exceptionHandler;

		@BeforeEach
		@SneakyThrows
		void mock() {
			when(xtaClientServiceFactory.create()).thenReturn(service);
			when(xtaExceptionHandlerFactory.create()).thenReturn(exceptionHandler);
		}

		@DisplayName("should call validate")
		@Test
		@SneakyThrows
		void shouldCallValidate() {
			factory.create();

			verify(configValidator).validate(config);
		}

		@DisplayName("should return")
		@Test
		@SneakyThrows
		void shouldReturn() {
			assertThat(factory.create())
					.usingRecursiveComparison()
					.isEqualTo(XtaClient.builder()
							.config(config)
							.service(service)
							.exceptionHandler(exceptionHandler)
							.build());
		}
	}
}