package de.ozgcloud.xta.client.core;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.xta.client.config.XtaClientConfig;
import de.ozgcloud.xta.client.mapper.RequestMapper;
import de.ozgcloud.xta.client.mapper.ResponseMapper;
import lombok.SneakyThrows;

class WrappedXtaServiceFactoryTest {

	@Mock
	private XtaPortTripleFactory xtaServiceFactory;
	@Mock
	private RequestMapper requestMapper;
	@Mock
	private ResponseMapper responseMapper;
	@Mock
	@SuppressWarnings("unused")
	private XtaClientConfig config;

	@InjectMocks
	private WrappedXtaServiceFactory factory;

	@DisplayName("create")
	@Nested
	class TestCreate {

		@Mock
		private XtaPortTriple ports;

		@BeforeEach
		@SneakyThrows
		void mock() {
			when(xtaServiceFactory.create()).thenReturn(ports);
		}

		@DisplayName("should have service")
		@Test
		@SneakyThrows
		void shouldHaveService() {
			var wrappedService = factory.create();

			assertThat(wrappedService.getPorts()).isEqualTo(ports);
		}

		@DisplayName("should have request mapper")
		@Test
		@SneakyThrows
		void shouldHaveRequestMapper() {
			var wrappedService = factory.create();

			assertThat(wrappedService.getRequestMapper()).isEqualTo(requestMapper);
		}

		@DisplayName("should have response mapper")
		@Test
		@SneakyThrows
		void shouldHaveResponseMapper() {
			var wrappedService = factory.create();

			assertThat(wrappedService.getResponseMapper()).isEqualTo(responseMapper);
		}
	}
}