package de.ozgcloud.xta.client.extension;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.Node;
import org.apache.logging.log4j.core.config.Property;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;

@Plugin(name = "StaticStringList", category = Node.CATEGORY)
public class StaticStringListAppender extends AbstractAppender {

	private static final ConcurrentLinkedQueue<String> LOG_LINES = new ConcurrentLinkedQueue<>();

	protected StaticStringListAppender(String name, Filter filter,
			Layout<? extends Serializable> layout, boolean ignoreExceptions,
			Property[] properties) {
		super(name, filter, layout, ignoreExceptions, properties);
	}

	@PluginFactory
	public static StaticStringListAppender createAppender(@PluginAttribute("name") String name) {
		return new StaticStringListAppender(name, null, null, true, Property.EMPTY_ARRAY);
	}

	public static List<String> getLogLines() {
		return new ArrayList<>(LOG_LINES);
	}

	public static void clearLogLines() {
		LOG_LINES.clear();
	}

	@Override
	public void append(LogEvent event) {
		LOG_LINES.add(event.getMessage().getFormattedMessage());
	}
}
