package de.ozgcloud.xta.client.extension;

import static de.ozgcloud.xta.client.extension.XtaServerSetupExtensionTestUtil.*;

import java.util.Objects;

import org.junit.jupiter.api.extension.AfterAllCallback;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.ExtensionContext;
import org.testcontainers.utility.DockerImageName;

import de.ozgcloud.xta.client.XtaClient;
import de.ozgcloud.xta.client.XtaClientFactory;
import de.ozgcloud.xta.client.XtaTestServerContainer;
import de.ozgcloud.xta.client.config.XtaClientConfig;
import lombok.Getter;
import lombok.Setter;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

@Getter
@Setter
@Slf4j
public class XtaTestServerSetupExtension implements BeforeAllCallback, AfterAllCallback {

	private static final DockerImageName XTA_TEST_SERVER_IMAGE = DockerImageName.parse("docker.ozg-sh.de/xta-test-server")
			.withTag("1.6.0");

	private static final String JOHN_SMITH_KEYSTORE_PATH = "store/john-smith-client-cert-keystore.p12";
	private static final String JOHN_SMITH_KEYSTORE_PASSWORD = "password";

	private static final String XTA_TEST_SERVER_TRUSTSTORE_PATH = "store/xta-test-server-truststore.jks";
	private static final String XTA_TEST_SERVER_TRUSTSTORE_PASSWORD = "password";

	private XtaClient silentTestClient;
	private XtaClientConfig silentTestClientConfig;
	private XtaClientFactory clientFactory;
	private XtaTestServerContainer xtaServerContainer;
	private XtaClientConfig.KeyStore clientCertKeyStore;
	private XtaClientConfig.KeyStore trustStore;

	@Override
	@SneakyThrows
	public void beforeAll(ExtensionContext context) {
		if (xtaServerContainer != null) {
			return;
		}
		setupServer();
		setupClient();
	}

	@SneakyThrows
	private void setupClient() {
		clientCertKeyStore = XtaClientConfig.KeyStore.builder()
				.content(readBytesFromResource(JOHN_SMITH_KEYSTORE_PATH))
				.type("PKCS12")
				.password(JOHN_SMITH_KEYSTORE_PASSWORD.toCharArray())
				.build();
		trustStore = XtaClientConfig.KeyStore.builder()
				.content(readBytesFromResource(XTA_TEST_SERVER_TRUSTSTORE_PATH))
				.type("JKS")
				.password(XTA_TEST_SERVER_TRUSTSTORE_PASSWORD.toCharArray())
				.build();
		silentTestClientConfig = createSpecificClientConfigBuilder()
				.logSoapRequests(false)
				.logSoapResponses(false)
				.build();
		silentTestClient = XtaClient.from(silentTestClientConfig);
	}

	private void setupServer() {
		xtaServerContainer = new XtaTestServerContainer(XTA_TEST_SERVER_IMAGE);

		xtaServerContainer.start();
	}

	@Override
	@SneakyThrows
	public void afterAll(ExtensionContext context) {
		if (xtaServerContainer == null) {
			return;
		}
		xtaServerContainer.stop();
		xtaServerContainer = null;
	}

	@SneakyThrows
	public XtaClientConfig.XtaClientConfigBuilder createSpecificClientConfigBuilder() {
		return createClientConfigBuilder()
				.managementServiceUrl(xtaServerContainer.getManagementPortUrl())
				.sendServiceUrl(xtaServerContainer.getSendPortUrl())
				.msgBoxServiceUrl(xtaServerContainer.getMsgBoxPortUrl())
				.clientCertKeystore(clientCertKeyStore)
				.trustStore(trustStore);
	}

	@SneakyThrows
	private static byte[] readBytesFromResource(String resourcePath) {
		try (var inputStream = XtaTestServerSetupExtension.class.getClassLoader().getResourceAsStream(resourcePath)) {
			return Objects.requireNonNull(inputStream, "Expect class-resource at: " + resourcePath).readAllBytes();
		}
	}

}
