package de.ozgcloud.xta.client.factory;

import java.math.BigInteger;
import java.time.ZonedDateTime;
import java.util.GregorianCalendar;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import de.ozgcloud.xta.client.model.XtaMessageStatus;
import genv3.de.xoev.transport.xta.x211.MessageStatusType;
import genv3.de.xoev.transport.xta.x211.TransportReport;

public class TransportReportTestFactory {

	public static final ZonedDateTime REPORT_TIME = ZonedDateTime.now();
	private static final DatatypeFactory datatypeFactory;

	static {
		try {
			datatypeFactory = DatatypeFactory.newInstance();
		} catch (DatatypeConfigurationException e) {
			throw new RuntimeException(e);
		}
	}

	public static TransportReport create() {
		return createWithStatus(XtaMessageStatus.OPEN);
	}

	public static TransportReport createWithStatus(XtaMessageStatus status) {
		var transportReport = new TransportReport();

		transportReport.setReportTime(
				datatypeFactory.newXMLGregorianCalendar(GregorianCalendar.from(REPORT_TIME))
		);
		transportReport.setMessageMetaData(MessageMetaDataTestFactory.create());
		transportReport.setMessageStatus(createMessageStatus(status));

		return transportReport;
	}

	private static MessageStatusType createMessageStatus(XtaMessageStatus status) {
		var messageStatus = new MessageStatusType();
		messageStatus.setStatus(BigInteger.valueOf(status.getCode()));
		return messageStatus;
	}

}
