package de.ozgcloud.xta.client.factory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.codec.Resources;
import org.apache.commons.io.IOUtils;

import de.ozgcloud.xta.client.extension.MessageFileProcessor;
import lombok.Builder;
import lombok.Getter;

public class ZipFileTestFactory {
	@Builder
	@Getter
	public static class TestZipEntry {
		private String name;
		private byte[] content;
	}

	static byte[] createZipFile(List<TestZipEntry> testZipEntries) {
		var byteArrayOutputStream = new ByteArrayOutputStream();
		try (var zipOutputStream = new ZipOutputStream(byteArrayOutputStream)) {
			for (TestZipEntry entry : testZipEntries) {
				zipOutputStream.putNextEntry(new ZipEntry(entry.getName()));
				zipOutputStream.write(entry.getContent());
				zipOutputStream.closeEntry();
			}
		} catch (IOException e) {
			throw new RuntimeException("Failed to create zip file", e);
		}
		return byteArrayOutputStream.toByteArray();
	}

	public static byte[] createAttachmentZipFile(String attachmentResourceDirectory, MessageFileProcessor messageFileProcessor) {
		return createZipFile(listResourceFileNames(attachmentResourceDirectory)
				.map(subName -> {
							byte[] data;
							var messageFilePath = attachmentResourceDirectory + "/" + subName;
							try {
								data = IOUtils.toByteArray(Resources.getInputStream(messageFilePath));
							} catch (IOException e) {
								throw new RuntimeException(e);
							}
							return TestZipEntry.builder()
									.name(subName)
									.content(messageFileProcessor.process(messageFilePath, data))
									.build();

						}
				).toList());
	}

	static Stream<String> listResourceFileNames(String resourceDirectoryPath) {
		try (var inputStream = Resources.getInputStream(resourceDirectoryPath)) {
			return Arrays.stream(IOUtils.toString(inputStream, StandardCharsets.UTF_8).split("\n"));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}
