package de.ozgcloud.xta.client.xdomea.reader;

import static de.ozgcloud.xta.client.xdomea.reader.XdomeaValueReader.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import javax.xml.xpath.XPathFactory;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import lombok.SneakyThrows;

class XdomeaValueReaderFactoryTest {

	@InjectMocks
	@Spy
	private XdomeaValueReaderFactory factory;

	@Mock
	private XPathFactory xPathFactory;

	private static final String TEXT_XPATH_STRING = "xpath/expression";

	@DisplayName("create")
	@Nested
	class TestCreate {

		@Mock
		XmlValueReader procesIdReader;

		@Mock
		XmlValueReader messageTypeReader;

		@Mock
		XmlValueReader authorIdReader;

		@Mock
		XmlValueReader authorIdPrefixReader;

		@Mock
		XmlValueReader readerIdReader;

		@Mock
		XmlValueReader readerIdPrefixReader;

		@Mock
		XmlValueReader primaryDocumentNameReader;

		@DisplayName("should return")
		@Test
		@SneakyThrows
		void shouldReturn() {
			doReturn(procesIdReader).when(factory).createXmlValueReader(PROCESS_ID_XPATH);
			doReturn(messageTypeReader).when(factory).createXmlValueReader(MESSAGE_TYPE_ID_SUFFIX_XPATH);
			doReturn(authorIdReader).when(factory).createXmlValueReader(AUTHOR_ID_SUFFIX_XPATH);
			doReturn(authorIdPrefixReader).when(factory).createXmlValueReader(AUTHOR_ID_PREFIX_XPATH);
			doReturn(readerIdReader).when(factory).createXmlValueReader(READER_ID_SUFFIX_XPATH);
			doReturn(readerIdPrefixReader).when(factory).createXmlValueReader(READER_ID_PREFIX_XPATH);
			doReturn(primaryDocumentNameReader).when(factory).createXmlValueReader(PRIMARY_DOCUMENT_NAME_XPATH);

			var reader = factory.create();

			assertThat(reader).usingRecursiveComparison().isEqualTo(XdomeaValueReader.builder()
					.xmlValueReaders(Map.of(
							PROCESS_ID_XPATH, procesIdReader,
							MESSAGE_TYPE_ID_SUFFIX_XPATH, messageTypeReader,
							AUTHOR_ID_SUFFIX_XPATH, authorIdReader,
							AUTHOR_ID_PREFIX_XPATH, authorIdPrefixReader,
							READER_ID_SUFFIX_XPATH, readerIdReader,
							READER_ID_PREFIX_XPATH, readerIdPrefixReader,
							PRIMARY_DOCUMENT_NAME_XPATH, primaryDocumentNameReader
					)).build());
		}
	}

	@DisplayName("create xml value reader factory")
	@Nested
	class TestCreateXmlValueReaderFactory {

		@DisplayName("should return")
		@Test
		void shouldReturn() {
			var reader = factory.createXmlValueReaderFactory(TEXT_XPATH_STRING);

			assertThat(reader).usingRecursiveComparison().isEqualTo(XmlValueReaderFactory.builder()
					.xPathFactory(xPathFactory)
					.textXpathString(TEXT_XPATH_STRING)
					.build());
		}
	}

	@DisplayName("create xml value reader")
	@Nested
	class TestCreateXmlValueReader {

		@Mock
		private XmlValueReaderFactory xmlValueReaderFactory;

		@Mock
		private XmlValueReader xmlValueReader;

		@DisplayName("should return")
		@Test
		void shouldReturn() {
			doReturn(xmlValueReaderFactory).when(factory).createXmlValueReaderFactory(TEXT_XPATH_STRING);
			when(xmlValueReaderFactory.create()).thenReturn(xmlValueReader);

			var result = factory.createXmlValueReader(TEXT_XPATH_STRING);

			assertThat(result).isEqualTo(xmlValueReader);
		}
	}

}